﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AIOUSBNetFragment;


namespace WindowsFormsApplication1
{
    

    public partial class Form1 : Form
    {
        public const byte PWM_RES = 200; // 200 divisor of global tick gives 1Hz frequency and 0.5% duty-cycle resolution
        UInt32 DeviceIndex = 0;
        byte[] pwm_struct = { PWM_RES, 
                                    0, // digital output index 0, port A bit 0, which is PIN 47 on the 50-pin header
                                    0, // initial duty cycle of zero
                                    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // seven more channels of bit, duty cycle (unused)
                                    1 // number of PWM bit entries in the array that are "active"; extras are ignored.
                                };
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            UInt32 devicesMask = AIOUSB.GetDevices();

            trackDutyCycle.Maximum = PWM_RES;

            if (devicesMask == 0)
            {
                labelBoardDetected.Text = "No board found, please resolve and re-run the sample";
                buttonInit.Enabled = false;
            }
            else
            while (devicesMask != 0 ) {
                uint PID=0, dio, ctr;
                string cn = @"this is a test board name";
                int ns = cn.Length;
                while (((devicesMask & 1) == 0) && (devicesMask != 0)) { DeviceIndex++; devicesMask = devicesMask / 2; } // find the first set bit & index

                uint Status = AIOUSB.QueryDeviceInfo(DeviceIndex, ref PID, ref ns, ref cn, out dio, out ctr);
                    if (Status == 0)
                        if (((PID >= 0x8001) && (PID<=0x8006)) // DIO
                            ||
                           ((PID >= 0x8010) && (PID <= 0x801F)) // IIRO / IDIO
                            ||
                           ((PID >= 0x8032) && (PID <= 0x803F))) // DIO
                        { labelBoardDetected.Text = cn; return; }
                        else
                            labelBoardDetected.Text = Status.ToString();
                DeviceIndex++; devicesMask = devicesMask / 2;
            }
        }

        private void buttonInit_Click(object sender, EventArgs e)
        {
            UInt32 status;
            byte[] outs =  {0x0F}; // all four ports set as output
            byte tris =  0x00; // no tristate
            byte[] digdata = { 0xff, 0xff, 0xff, 0xff}; // all bits high at start
            status = AIOUSB.DIO_Configure(DeviceIndex, tris, outs, digdata);
            if (status==0)
                status = AIOUSB.GenericVendorWrite(DeviceIndex, 0x3C, 60000, 0x8001, 0, null); // set global tick to 200Hz
            labelStatus.Text = status.ToString();
            if (status==0)
                trackDutyCycle.Visible=true;
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            UInt32 status;
            labelDutyCycle.Text = (trackDutyCycle.Value * 100.0 / PWM_RES ).ToString() + " Duty Cycle %";
            pwm_struct[2] = (byte)trackDutyCycle.Value;
            status = AIOUSB.GenericVendorWrite(DeviceIndex, 0x3C, 0, 0x0071, 18, pwm_struct);
            labelStatus.Text = status.ToString();
        }
    }
}

namespace AIOUSBNetFragment
{
    public class AIOUSB
    {
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 QueryDeviceInfo(UInt32 DeviceIndex, ref UInt32 pPID, ref Int32 pNameSize, IntPtr Name, ref UInt32 pDIOBytes, ref UInt32 pCounters);

        public static UInt32 QueryDeviceInfo(UInt32 DeviceIndex, ref UInt32 pPID, ref Int32 pNameSize, ref string Name, out UInt32 pDIOBytes, out UInt32 pCounters)
        {
            UInt32 Status;
            pDIOBytes = 0; pCounters = 0;
            IntPtr pName = Marshal.AllocHGlobal(pNameSize);

            Status = QueryDeviceInfo(DeviceIndex, ref pPID, ref pNameSize, pName, ref pDIOBytes, ref pCounters);
            Name = Marshal.PtrToStringAnsi(pName);
            Name = Name.Substring(0, Math.Min(Name.Length, pNameSize)); // unsure....
            return Status;
        }
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_Configure(UInt32 DeviceIndex, Byte Tristate, ref UInt32 OutMask, ref UInt32 Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_Configure(UInt32 DeviceIndex, Byte Tristate, ref Int16 OutMask, [In] Byte[] Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_Configure(UInt32 DeviceIndex, Byte Tristate, [In] Byte[] OutMask, [In] Byte[] Data);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_Configure(UInt32 DeviceIndex, Byte Tristate, ref Byte OutMask, ref Byte Data);


        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_ConfigureMasked(UInt32 DeviceIndex, [In] Byte[] pOuts, [In] Byte[] pOutsMask, [In] Byte[] pData, [In] Byte[] pDataMask, [In] Byte[] pTristates, [In] Byte[] pTristatesMask);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamOpen(UInt32 DeviceIndex, UInt32 bIsRead);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamClose(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 GetDevices();
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 ClearDevices();
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 AIOUSB_CloseDevice(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 ResolveDeviceIndex(UInt32 DeviceIndex);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_ConfigureEx(UInt32 DeviceIndex, [In] Byte[] pOutMask, [In] Byte[] pData, [In] Byte[] pTristateMask);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamSetClocks(UInt32 DeviceIndex, ref Double ReadClockHz, ref Double WriteClockHz);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamFrame(UInt32 DeviceIndex, UInt32 FramePoints, byte[] pFrameData, ref UInt32 ByteTransferred);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 DIO_StreamFrame(UInt32 DeviceIndex, UInt32 FramePoints, UInt16[] pFrameData, ref UInt32 ByteTransferred);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 AIOUSB_UploadD15LoFirmwaresByPID(string PIDsvsPaths);
        [DllImport("AIOUSB.DLL", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl)]
        public static extern UInt32 GenericVendorWrite(UInt32 DeviceIndex, Byte Request, UInt16 Value, UInt16 Index, UInt32 DataSize, [In] byte[] pData);
    }
}
